#include <ardour/route.h>

#include "automation_time_axis.h"
#include "automation_line.h"
#include "public_editor.h"
#include "canvas-simplerect.h"
#include "rgb_macros.h"

#include "i18n.h"

using namespace ARDOUR;
using namespace Gtk;

static const gchar * small_x_xpm[] = {
"11 11 2 1",
" 	c None",
".	c #FFFFFF",
"           ",
"           ",
"  .     .  ",
"   .   .   ",
"    . .    ",
"     .     ",
"    . .    ",
"   .   .   ",
"  .     .  ",
"           ",
"           "};

AutomationTimeAxisView::AutomationTimeAxisView (Session& s, Route& r, PublicEditor& e, Widget* p, std::string nom, std::string nomparent)
	: AxisView (s), 
	  TimeAxisView (s, e, p),
	  route (r),
	  _name (nom),
	  height_button (_("h")),
	  clear_button (_("clear")),
	  active_button (_("active"))
{
	base_rect = gtk_canvas_item_new (GTK_CANVAS_GROUP(canvas_display),
					 gtk_canvas_simplerect_get_type(),
					 "x1", 0.0,
					 "y1", 0.0,
					 "x2", 1000000.0,
					 "outline_color_rgba", RGBA_TO_UINT(0,0,0,255),
					 /* outline ends and bottom */
					 "outline_what", (guint32) (0x1|0x2|0x8),
					 "fill_color_rgba", RGBA_TO_UINT(227,247,238,145),
					 NULL);

	gtk_signal_connect (GTK_OBJECT(base_rect), "event",
			    (GtkSignalFunc) PublicEditor::canvas_automation_track_event,
			    this);

	name_label.set_text (_name);

	hide_button.add (*(manage (new Pixmap (small_x_xpm))));

	height_button.set_name ("TrackSizeButton");
	active_button.set_name ("TrackVisualButton");
	clear_button.set_name ("TrackVisualButton");
	hide_button.set_name ("TrackRemoveButton");

	/* rearrange the name display */

	/* we never show these for automation tracks, so make
	   life easier and remove them.
	*/

	name_hbox.remove (name_entry);
	name_hbox.remove (expansion_box);

	/* move the name label over a bit */

	name_label.set_alignment (1.0, 0.5);

	Label* plugname;

	if (nomparent.length()) {

		/* limit the plug name string */

		string pname = nomparent;

		if (pname.length() > 16) {
			pname = pname.substr (0, 13);
			pname += "...";
		}

		plugname = manage (new Label (pname));
		plugname->set_name (X_("TrackPlugName"));
		plugname->set_alignment (1.0, 0.5);
		name_label.set_name (X_("TrackParameterName"));
		controls_table.remove (name_hbox);
		controls_table.attach (*plugname, 0, 5, 0, 1, GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND);
		controls_table.attach (name_hbox, 0, 5, 1, 2, GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND);
	}

	/* add the buttons */

	controls_table.attach (hide_button, 6, 7, 0, 1, GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND);
	controls_table.attach (height_button, 6, 7, 1, 2, GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND);

	controls_table.attach (active_button, 7, 9, 0, 1, GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND);
	controls_table.attach (clear_button, 7, 9, 1, 2, GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND);

	controls_table.show_all ();

	height_button.clicked.connect (slot (*this, &AutomationTimeAxisView::height_clicked));
	clear_button.clicked.connect (slot (*this, &AutomationTimeAxisView::clear_clicked));
	hide_button.clicked.connect (slot (*this, &AutomationTimeAxisView::hide_clicked));

	line = 0;
}

AutomationTimeAxisView::~AutomationTimeAxisView ()
{
	if (line) {
		delete line;
	}
}

void
AutomationTimeAxisView::height_clicked ()
{
	popup_size_menu (0);
}

void
AutomationTimeAxisView::clear_clicked ()
{
	line->clear ();
}

void
AutomationTimeAxisView::set_height (TrackHeight h)
{
	bool changed = (height != (unsigned int) h);

	controls_table.show_all ();

	TimeAxisView::set_height (h);
	gtk_object_set (GTK_OBJECT(base_rect), "y2", (double) h, NULL);
	line->set_height (h);


	name_entry.hide ();
	expansion_box.hide ();

	if (changed) {
		/* only emit the signal if the height really changed */
		EMIT_SIGNAL route.gui_changed ("track_height", (void *) 0);
	}
}

void
AutomationTimeAxisView::set_samples_per_unit (double spu)
{
	TimeAxisView::set_samples_per_unit (editor.get_current_zoom());
	line->reset ();
}
 
void
AutomationTimeAxisView::hide_clicked ()
{
	hide ();
}


