#ifndef __ardour_gtk_automation_time_axis_h__
#define __ardour_gtk_automation_time_axis_h__

#include <string>
#include <ardour/types.h>

#include "time_axis_view.h"

namespace ARDOUR {
	class Session;
	class Route;
}

namespace Gtk {
	class Widget;
}

class PublicEditor;
class TimeSelection;
class AudioRegionSelection;
class AutomationLine;

class AutomationTimeAxisView : public TimeAxisView {
  public:
	AutomationTimeAxisView (ARDOUR::Session&,
				ARDOUR::Route&,
				PublicEditor&,
				Gtk::Widget* parent,
				std::string name,
				std::string plug_name = "");

	~AutomationTimeAxisView();
	
	void set_height (TimeAxisView::TrackHeight);
	void set_samples_per_unit (double);
	std::string name() const { return _name; }

	virtual void add_automation_event (GtkCanvasItem *item, GdkEvent *event, jack_nframes_t, double) = 0;

	AutomationLine* line;

	void show_selection (TimeSelection&) {}
	void hide_selection () {}
	void reshow_selection (TimeSelection&) {}
	void set_selected_regionviews (AudioRegionSelection&) {}
	void show_timestretch (TimeSelection&) {}
	void hide_timestretch () {}

  protected:
	ARDOUR::Route& route;
	GtkCanvasItem* base_rect;
	string _name;

	Gtk::Button        hide_button;
	Gtk::Button        height_button;
	Gtk::Button        clear_button;
	Gtk::ToggleButton  active_button; 

	void clear_clicked ();
	void height_clicked ();
	void hide_clicked ();
	void active_toggled ();

};

#endif /* __ardour_gtk_automation_time_axis_h__ */
