#include <ardour/curve.h>
#include <ardour/audioregion.h>

#include "public_editor.h"
#include "region_gain_line.h"
#include "regionview.h"
#include "utils.h"

#include <ardour/session.h>

using namespace ARDOUR;

AudioRegionGainLine::AudioRegionGainLine (string name, Session& s, AudioRegionView& r, PublicEditor& e, GtkCanvasItem* parent,
					  Curve& c, 
					  gint (*point_callback)(GtkCanvasItem*, GdkEvent*, gpointer),
					  gint (*line_callback)(GtkCanvasItem*, GdkEvent*, gpointer))
	: AutomationLineCurve (name, e, parent, c, point_callback, line_callback),
	  session (s),
	  rv (r)
{
	set_verbose_cursor_uses_gain_mapping (true);
	terminal_points_can_slide = false;
}

UndoAction
AudioRegionGainLine::get_memento()
{
	return rv.region.get_memento();
}

void
AudioRegionGainLine::view_to_model_y (double& y)
{
	y = slider_position_to_gain (y);
	y = max (0.0, y);
	y = min (2.0, y);
}

void
AudioRegionGainLine::model_to_view_y (double& y)
{
	y = gain_to_slider_position (y);
}

void
AudioRegionGainLine::change_model (unsigned long index, double x, double y)
{
	rv.region.modify_envelope_control_point (index, (jack_nframes_t) x, y);
}

void
AudioRegionGainLine::change_model_range (unsigned long start, unsigned long end, double delta)
{
	rv.region.modify_envelope_control_points (start, end, delta);
}

void
AudioRegionGainLine::remove_model_range (unsigned long start, unsigned long end)
{
	rv.region.remove_envelope_control_points (start, end);
}

