/*
    Copyright (C) 2001 Paul Davis 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: regionview.h,v 1.50 2003/11/21 04:47:44 pbd Exp $
*/

#ifndef __gtk_ardour_region_view_h__
#define __gtk_ardour_region_view_h__

#include <vector>
#include <gtk--.h>
#include <gtk-canvas.h>
#include <sigc++/signal_system.h>
#include <ardour/region.h>

#include "time_axis_view_item.h"
#include "automation_line.h"
#include "enums.h"

namespace ARDOUR {
	class AudioRegion;
};

class AudioTimeAxisView;
class AudioRegionGainLine;
class AudioRegionEditor;

struct AudioRegionView : public TimeAxisViewItem
{
    AudioRegionView (GtkCanvasGroup *, 
		     AudioTimeAxisView&,
		     ARDOUR::AudioRegion&,
		     double initial_samples_per_unit,
		     double amplitude_above_axis,
		     GdkColor& base_color);
    ~AudioRegionView ();

    ARDOUR::AudioRegion& region;  // ok, let 'em have it
    bool is_valid() const { return valid; }
    void set_valid (bool yn) { valid = yn; }

    std::string get_item_name();
    void set_height (double);
    void set_samples_per_unit (double);

    void set_amplitude_above_axis (gdouble spp);

    void raise ();
    void raise_to_top ();
    void lower ();
    void lower_to_bottom ();
    
    void set_envelope_visible (bool);
    void set_waveform_visible (bool yn);
    void set_waveform_shape (WaveformShape);

    bool waveform_rectified() const { return _flags & WaveformRectified; }
    bool waveform_visible() const { return _flags & WaveformVisible; }
    bool envelope_visible() const { return _flags & EnvelopeVisible; }
    
    void show_region_editor ();
    void hide_region_editor();

    void add_gain_point_event (GtkCanvasItem *item, GdkEvent *event);
    void remove_gain_point_event (GtkCanvasItem *item, GdkEvent *event);

    AudioRegionGainLine* get_gain_line() const { return gain_line; }

    void region_changed (ARDOUR::Change);

    static SigC::Signal1<void,AudioRegionView*> GoingAway;

  private:
    enum Flags {
	    EnvelopeVisible = 0x1,
	    WaveformVisible = 0x4,
	    WaveformRectified = 0x8
    };

    vector<GtkCanvasItem *> waves; /* waveviews */
    GtkCanvasItem *sync_mark; /* polgyon for sync position */
    GtkCanvasItem *no_wave_msg; /* text */
    GtkCanvasItem *zero_line; /* simpleline */

    AudioRegionGainLine* gain_line;
    AudioRegionEditor *editor;

    vector<ControlPoint *> control_points;
    double _amplitude_above_axis;
    double current_visible_sync_position;

    unsigned int _flags;
    unsigned int wave_color;
    unsigned int muted_wave_color;
    unsigned int gain_color;
    bool     valid; /* see StreamView::redisplay_diskstream() */

    void region_resized (ARDOUR::Change);
    void region_moved (void *);
    void region_muted ();
    void region_opacity ();
    void region_layered ();
    void region_renamed ();
    void region_sync_changed ();

    void create_wave ();
    void peaks_ready_handler ();
    void reset_name (gdouble width);

    void set_flags (XMLNode *);
    void store_flags ();

    void set_colors ();
    void compute_colors (GdkColor&);
    void set_frame_color ();
    void reset_width_dependent_items (double pixel_width);
};

#endif /* __gtk_ardour_region_view_h__ */
