/*
    Copyright (C) 2000-2003 Paul Davis 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: selection.h,v 1.9 2003/10/29 04:27:22 pbd Exp $
*/

#ifndef __ardour_gtk_selection_h__
#define __ardour_gtk_selection_h__

#include <sigc++/signal_system.h>

#include "time_selection.h"
#include "region_selection.h"
#include "track_selection.h"
#include "automation_selection.h"

class AutomationLine;
class TimeAxisView;
class AudioRegionView;

class Selection : public SigC::Object 
{
  public:
	TrackSelection       tracks;
	AudioRegionSelection audio_regions;
	TimeSelection        time;
	AutomationSelection  lines;

	Selection() {
		next_time_id = 0;
		clear();
	}

	Selection& operator= (const Selection& other);

	SigC::Signal0<void> RegionsChanged;
	SigC::Signal0<void> RoutesChanged;
	SigC::Signal0<void> TimeChanged;
	SigC::Signal0<void> LinesChanged;

	void clear ();
	
	void dump_region_layers();
 
	void set (TimeAxisView*);
	void set (AudioRegionView*);
	long set (TimeAxisView*, jack_nframes_t, jack_nframes_t);
	void set (AutomationLine*);
	
	void add (TimeAxisView*);
	void add (AudioRegionView*);
	long add (jack_nframes_t, jack_nframes_t);
	void add (AutomationLine*);
	
	void remove (TimeAxisView*);
	void remove (AudioRegionView*);
	void remove (unsigned long selection_id);
	void remove (jack_nframes_t, jack_nframes_t);
	void remove (AutomationLine*);

	void replace (unsigned long time_index, jack_nframes_t start, jack_nframes_t end);
	
	void clear_audio_regions();
	void clear_tracks ();
	void clear_time();
	void clear_lines ();
	
	void foreach_audio_region (void (ARDOUR::AudioRegion::*method)(void));
	void foreach_audio_region (void (ARDOUR::Region::*method)(void));
	template<class A> void foreach_audio_region (void (ARDOUR::AudioRegion::*method)(A), A arg);
	template<class A> void foreach_audio_region (void (ARDOUR::Region::*method)(A), A arg);

#if 0
	template<class A1, class A2> void foreach_region (void (AudioRegionView::*method)(A1, A2), A1 arg1, A2 arg2);
	void foreach_route (void (TimeAxisView::*method)(void));
	template<class A> void foreach_route (void (TimeAxisView::*method)(A), A arg);
	template<class A1, class A2> void foreach_route (void (TimeAxisView::*method)(A1, A2), A1 arg1, A2 arg2);
#endif

  private:
	unsigned long next_time_id;
};

bool operator==(const Selection& a, const Selection& b);

#endif /* __ardour_gtk_selection_h__ */
